/* Arithmetique des polynomes,  1 variable, denses, coeff par ordre croissant
   Les polynomes seront toujours normalises (dernier coeff non nul)
   Mode d'emploi, par exemple pour des polynomes  coeff rationnels: 
   #include "poly.hh" 
   #include <gmpxx.h>
   typedef vector<mpq_class> poly;
   puis s'utilise comme int avec
   les operations +,-,*,/,% et pour les entrees/sorties << >>
   Format pour les vecteurs [ coeff0 coeff1 ... coeffn ]
   On peut aussi separer les coefficients par des virgules
   Attention la division / doit renvoyer un quotient exact, ce n'est pas le cas
   pour les entiers, il faut alors effectuer une pseudo-division
   Pour T=modulaire, il faut auparavant remplacer q.back() par 1
   Il faut alors specialiser /
*/

#include <iostream>
#include <vector>
#include <algorithm>
#include <stdexcept>

template <class T> class dbgprint_vector: public std::vector<T> {
public:
  // inherited constructors
  dbgprint_vector() : std::vector<T>::vector() { };
  dbgprint_vector(const typename std::vector<T>::const_iterator & b,const typename std::vector<T>::const_iterator & e) : std::vector<T>::vector(b,e) { };
  dbgprint_vector(int i) : std::vector<T>::vector(i) { };
  dbgprint_vector(int i,const T & t) : std::vector<T>::vector(i,t) { };
  // ~dbgprint_vector() { };
  // inherited destructors
  void dbgprint() { std::cout << *this << std::endl; }
};


// Supprime les 0 non significatifs
template<class T>
void trim(dbgprint_vector<T> & v){
  for (;!v.empty();){
    if (v.back()!=0)
      break;
    v.pop_back();
  }
}

template<class T>
dbgprint_vector<T> operator +(const dbgprint_vector<T> & p,const dbgprint_vector<T> & q){
  typename dbgprint_vector<T>::const_iterator it=p.begin(),itend=p.end(),jt=q.begin(),jtend=q.end();
  dbgprint_vector<T> res;
  res.reserve(std::max<int>(itend-it,jtend-jt));
  for (;it!=itend && jt!=jtend;++it,++jt){
    res.push_back((*it)+(*jt));
  }
  for (;it!=itend;++it)
    res.push_back(*it);
  for (;jt!=jtend;++jt)
    res.push_back(*jt);
  trim(res);
  return res;
}

template<class T>
dbgprint_vector<T> operator -(const dbgprint_vector<T> & p){
  dbgprint_vector<T> res(p);
  typename dbgprint_vector<T>::iterator it=res.begin(),itend=res.end();
  for (;it!=itend;++it)
    *it=-*it;
  return res;
}

template<class T>
dbgprint_vector<T> operator -(const dbgprint_vector<T> & p,const dbgprint_vector<T> & q){
  return p + (-q); 
}

template<class T>
dbgprint_vector<T> operator *(const dbgprint_vector<T> & p,const dbgprint_vector<T> & q){
  int pd=p.size(),qd=q.size();
  if (!pd)
    return p;
  if (!qd)
    return q;
  dbgprint_vector<T> res(pd+qd-1);
  for (int i=0;i<pd;++i){
    for (int j=0;j<qd;++j)
      res[i+j] = res[i+j] + p[i]*q[j];
  }
  return res;
}

template<class T>
dbgprint_vector<T> operator *(const T & p,const dbgprint_vector<T> & q){
  dbgprint_vector<T> res(q);
  typename dbgprint_vector<T>::iterator it=res.begin(),itend=res.end();
  for (;it!=itend;++it)
    *it = p * (*it);
  return res;  
}

// Attention: d >=0 !
template<class T>
dbgprint_vector<T> fois_shift(const T & p,const dbgprint_vector<T> & q,int d){
  if (d<0)
    throw(std::runtime_error("Erreur: decalage negatif dans fois_shift"));
  dbgprint_vector<T> res(q.size()+d);
  typename dbgprint_vector<T>::iterator it=res.begin(),itend=res.end();
  typename dbgprint_vector<T>::const_iterator jt=q.begin();
  for (it+=d;it!=itend;++it,++jt)
    *it = p * (*jt);
  return res;  
}

template<class T>
void divrem(dbgprint_vector<T> & quo,dbgprint_vector<T> & rem,const dbgprint_vector<T> & p,const dbgprint_vector<T> & q){
  quo.clear();
  rem=p;
  int qd=q.size(),rd=rem.size();
  if (qd==0 || rd<qd)
    return;
  T b=q.back();
  quo=dbgprint_vector<T>(rd-qd+1);
  for (;(rd=rem.size())>=qd;){
    T m=rem.back()/b;
    if (m==0)
      throw(std::runtime_error("Erreur: quotient=0 dans dbgprint_vector<T> /"));
    quo[rd-qd]=m;
    rem = rem - fois_shift(m,q,rd-qd);
  }
}

template<class T>
dbgprint_vector<T> operator /(const dbgprint_vector<T> & p,const dbgprint_vector<T> & q){
  dbgprint_vector<T> quo,rem;
  divrem(quo,rem,p,q);
  return quo;
}

template<class T>
dbgprint_vector<T> operator %(const dbgprint_vector<T> & p,const dbgprint_vector<T> & q){
  dbgprint_vector<T> quo,rem;
  divrem(quo,rem,p,q);
  return rem;
}

template<class T>
std::ostream & operator << (std::ostream & o ,const dbgprint_vector<T> & q){
  typename dbgprint_vector<T>::const_iterator it=q.begin(),itend=q.end();
  o << "[ " ;
  for (;it!=itend;++it){
    o << *it << " " ;
  }
  o << "] ";
  return o;
}

template<class T>
std::istream & operator >> (std::istream & i,dbgprint_vector<T> & q){
  q.clear();
  char ch;
  for (;!i.eof();){
    i.get(ch);
    if (ch==' '  || ch=='\n')
      continue;
    if (ch=='[')
      break;
    throw(std::runtime_error("Polynome doit commencer par ["));
  }
  for (; ;){
    if (i.eof())
      throw(std::runtime_error("Fin du flux atteint en entree"));
    i.get(ch);
    if (ch==']')
      return i;
    if (ch==' ' || ch=='\n' || ch==',')
      continue;
    i.putback(ch);
    T tmp;
    i >> tmp;
    q.push_back(tmp);
  }
}
